#ifndef MMA8452_H__
#define MMA8452_H__

#include "i2c.h"  // not the wire library, can't use pull-ups
#include "Arduino.h"



//MMA8451Q Registers

#define STATUS            0x00		// STATUS Register 

#define OUT_X_MSB         0x01		// [7:0] are 8 MSBs of the 14-bit X-axis sample
#define OUT_X_LSB         0x02		// [7:2] are the 6 LSB of 14-bit X-axis sample
#define OUT_Y_MSB         0x03		// [7:0] are 8 MSBs of the 14-bit Y-axis sample
#define OUT_Y_LSB          0x04		// [7:2] are the 6 LSB of 14-bit Y-axis sample
#define OUT_Z_MSB         0x05		// [7:0] are 8 MSBs of the 14-bit Z-axis sample
#define OUT_Z_LSB        0x06		// [7:2] are the 6 LSB of 14-bit Z-axis sample

#define F_SETUP           0x09    	// F_SETUP FIFO Setup Register 
#define TRIG_CFG          0x0A    	// TRIG_CFG Map of FIFO data capture events 
#define SYSMOD            0x0B    	// SYSMOD System Mode Register 
#define INT_SOURCE        0x0C    	// INT_SOURCE System Interrupt Status Register 
#define WHO_AM_I          0x0D    	// WHO_AM_I Device ID Register 
#define XYZ_DATA_CFG      0x0E    	// XYZ_DATA_CFG Sensor Data Configuration Register 
#define HP_FILTER_CUTOFF  0x0F    	// HP_FILTER_CUTOFF High Pass Filter Register 

#define PL_STATUS         0x10    	// PL_STATUS Portrait/Landscape Status Register 
#define PL_CFG            0x11    	// PL_CFG Portrait/Landscape Configuration Register 
#define PL_COUNT          0x12    	// PL_COUNT Portrait/Landscape Debounce Register 
#define PL_BF_ZCOMP       0x13    	// PL_BF_ZCOMP Back/Front and Z Compensation Register 
#define P_L_THS           0x14    	// P_L_THS Portrait to Landscape Threshold Register 

#define FF_MT_CFG         0x15    	// FF_MT_CFG Freefall and Motion Configuration Register 
#define FF_MT_SRC         0x16    	// FF_MT_SRC Freefall and Motion Source Register 
#define FF_MT_THS         0x17    	// FF_MT_THS Freefall and Motion Threshold Register 
#define FF_MT_COUNT       0x18    	// FF_MT_COUNT Freefall Motion Count Register 

#define TRANSIENT_CFG     0x1D    	// TRANSIENT_CFG Transient Configuration Register 
#define TRANSIENT_SRC     0x1E    	// TRANSIENT_SRC Transient Source Register 
#define TRANSIENT_THS     0x1F    	// TRANSIENT_THS Transient Threshold Register 
#define TRANSIENT_COUNT   0x20    	// TRANSIENT_COUNT Transient Debounce Counter Register 

#define PULSE_CFG         0x21    	// PULSE_CFG Pulse Configuration Register 
#define PULSE_SRC         0x22    	// PULSE_SRC Pulse Source Register 
#define PULSE_THSX        0x23    	// PULSE_THS XYZ Pulse Threshold Registers 
#define PULSE_THSY        0x24
#define PULSE_THSZ        0x25
#define PULSE_TMLT        0x26    	// PULSE_TMLT Pulse Time Window Register 
#define PULSE_LTCY        0x27    	// PULSE_LTCY Pulse Latency Timer Register 
#define PULSE_WIND        0x28    	// PULSE_WIND Second Pulse Time Window Register 

#define ASLP_COUNT        0x29    	// ASLP_COUNT Auto Sleep Inactivity Timer Register 

#define CTRL_REG1             0x2A    	// CTRL_REG1 System Control 1 Register 
#define CTRL_REG2             0x2B    	// CTRL_REG2 System Control 2 Register 
#define CTRL_REG3             0x2C    	// CTRL_REG3 Interrupt Control Register 
#define CTRL_REG4             0x2D    	// CTRL_REG4 Interrupt Enable Register 
#define CTRL_REG5             0x2E    	// CTRL_REG5 Interrupt Configuration Register 

#define OFF_X_REG             0x2F    	// XYZ Offset Correction Registers 
#define OFF_Y_REG             0x30
#define OFF_Z_REG             0x31


// Iniitial values of Control Registers
//REG1
#define ASLP_50_HZ  0x00 //50Hz en mode sleep
#define ODR_100_HZ  0x18  //100Hz en mode eveil
#define LNOISE 0x04
#define F_READ 0x02
#define ACTIVE 0x01
//REG2
#define SLPE  0x04 
#define SMODS_LOW_POW  0x00 //sur les bits 4 3
#define MODS_NORMAL_POW  0x00//sur les bits 1 0
#define ST  0x80
#define RST  0x40
//REG3
#define WAKE_TRANS  0x40
#define WAKE_LNDPRT 0x20
#define WAKE_PULSE 0x10
#define WAKE_FF_MT  0x08 
#define IPOL 0x02
#define PP_OD 0x01
//REG4
#define INT_EN_ASLP  0x80
#define INT_EN_TRANS 0x20
#define INT_EN_LNDPRT  0x10
#define INT_EN_PULSE 0x08
#define INT_EN_FF_MT  0x04
#define INT_EN_DRDY_  0x01
//REG5
#define INT_CFG_ASLP 0x80
#define INT_CFG_TRANS  0x20
#define INT_CFG_LNDPRT  0x10
#define INT_CFG_PULSE 0x08
#define INT_CFG_FF_MT  0x04
#define INT_CFG_DRDY_  0x01

//Auto Wake/Sleep Detection
#define ASLP_COUNT_50_HZ_960_MS  0x03

//Motion confifuration register
#define ELE_MOTION  0x00
#define OAE_MOTION  0x40
#define XYZ_EFE 0x10

#define DBCNTM_MOTION  0x00
#define THS_0_2_G  0x34

#define FF_MT_DEBOUNCE  0xFF

//Transient configuration register
/*#define ELE  0x10
#define XYZ_EFE 0x0E
#define HPF_BYP 0x01

#define DBCNTM_TRANS  0x00
#define TRANS_THS_0_5_G  0x05
 
#define TRANS_DEBOUNCE_50_HZ_20_MS  0x03
*/


// Pin definitions
#define int1Pin 2  // These can be changed, 2 and 3 are the Arduinos ext int pin
#define int2Pin 3

//configure l'accelerometre
void setup_MMA8452();

//lit la valeur de l'acceleration
void readAccelData(int * destination);

// This function will read the status of the tap source register.
// Print if there's been a single or double tap, and on what axis.
void tapHandler();

// This function will read the p/l source register and
// print what direction the sensor is now facing
void portraitLandscapeHandler();

// Read a single byte from address and return it as a byte
byte readRegister(uint8_t address);

#endif
