#ifndef _spi_h__
#define _spi_h__

#include <stdint.h>
#include <avr/io.h>
#include <stdio.h>
#include <stdlib.h>
#include <stdbool.h>
#include <string.h>
#include "pins_arduino.h"


#ifdef __cplusplus
extern "C"{
#endif

// create alias for the different SPI chip pins - code assumes all on port B
#if (defined(__AVR_AT90USB82__) || defined(__AVR_AT90USB162__))
 #define SPI_SS_PIN PORTB0
 #define SPI_SCK_PIN PORTB1
 #define SPI_MOSI_PIN PORTB2
 #define SPI_MISO_PIN PORTB3
#elif (defined(__AVR_ATmega48__) || defined(_AVR_ATmega88__) || defined(__AVR_ATmega168__) || defined(__AVR_ATmega328__) || defined(__AVR_ATmega328P__))
 #define SPI_SCK_PIN 5
 #define SPI_MOSI_PIN 3
 #define SPI_MISO_PIN 4
#else
 #error unknown processor - add to spi.h
#endif

// SPI clock modes
#define SPI_MODE0 0x00 /* Sample (Rising) Setup (Falling) CPOL=0, CPHA=0 */
#define SPI_MODE1 0x04 /* Setup (Rising) Sample (Falling) CPOL=0, CPHA=1 */
#define SPI_MODE2 0x08 /* Sample (Falling) Setup (Rising) CPOL=1, CPHA=0 */
#define SPI_MODE3 0x0C /* Setup (Falling) Sample (Rising) CPOL=1, CPHA=1 */

// data direction
#define SPI_LSB 1 /* send least significant bit (bit 0) first */
#define SPI_MSB 0 /* send most significant bit (bit 7) first */

// whether to raise interrupt when data received (SPIF bit received)
#define SPI_NO_INTERRUPT 0
#define SPI_INTERRUPT 1

// slave or master with clock diviser
#define SPI_SLAVE 0xF0
#define SPI_CLOCK_DIV4 0x00 /* chip clock/4 */
#define SPI_CLOCK_DIV16 0x01 /* chip clock/16 */
#define SPI_CLOCK_DIV64 0x02 /* chip clock/64 */
#define SPI_CLOCK_DIV128 0x03 /* chip clock/128 */
#define SPI_CLOCK_DIV2 0x04 /* chip clock/2 */
#define SPI_CLOCK_DIV8 0x05 /* chip clock/8 */
#define SPI_CLOCK_DIV32 0x06 /* chip clock/32 */

#define SPI_MODE_MASK 0x0C  // CPOL = bit 3, CPHA = bit 2 on SPCR
#define SPI_CLOCK_MASK 0x03  // SPR1 = bit 1, SPR0 = bit 0 on SPCR
#define SPI_2XCLOCK_MASK 0x01  // SPI2X = bit 0 on SPSR

   void spi_begin(); // Default
   void spi_end();

   void spi_setBitOrder(uint8_t);
   void spi_setDataMode(uint8_t);
   void spi_setClockDivider(uint8_t);

   uint8_t spi_transfer(uint8_t);

// receive the byte of data waiting on the SPI buffer and
// set the next byte to transfer - for use in slave mode
// when interrupts are enabled.
uint8_t received_from_spi(uint8_t out);

#ifdef __cplusplus
} // extern "C"
#endif

#endif

